% VCplotCollection.m      MA2VC-MA3VC September 2015/2016, Reading
% Collection of all routines used to draw figures in Vector Calculus notes.
%
% Can be run interactively or take one choice as input
% (i.e. either type VCplotCollection and then choose from options offered
%  or type e.g. VCplotCollection(5); ),
%
% Slightly different results with different versions of Matlab, most
% pictures in notes were obtained with v2014, some with v2015. 
% Main differences are in color scheme and positioning of some text.
%
% Contains function    PlotMyMesh   to show change of variables.
%
% All figures are saved in .eps format.
%
%
%
% The main commands for plotting are 
%   plot            2d curves
%   plot3           3d curves
%   polar           curves in polar coordinates
%   contour         level sets for planar scalar fields
%   quiver          arrow plot for vector fields
%   streamslice     streamline plot for vector fields
%   surf            surfaces and graph (smooth plots)
%   mesh            surfaces and graph (grid plots)
% The field are evaluated on points lying on a Cartesian mesh built using
%   linspace
%   meshgrid
%
% Unfortunately streamslice is not available in Octave (and I am not aware
% of valid alternative commands), so options 12, 13 and 24 below don't work
% in Octave.
%
% Type "help command_name" in Matlab's terminal to understand how to use
% these commands. 
% See also some comments in this file, the first examples are better
% commented than the last ones.
%
% Don't be scared by the length of this file: each figure corresponds to a
% "case" of few lines that can be understood independently of the rest.
%
% (Many lines are only maquillage to get nice pictures in the notes.)


function VCplotCollection(varargin)
format compact; close all;

%% choose which plot
if nargin==0    % if no input is given, ask:
    disp('What do you want to plot?  Choose your option:');
    disp(' CURVES');
    disp('  1 - Helix');
    disp('  2 - 3-leaved clover');
    disp('  3 - Planar cubic');
    disp('  4 - Curves of revision exercises for section 1');
    disp('  5 - 3 paths for line-integration of vector fields');
    disp('  6 - Polar curves');
    disp('  7 - Curve in spherical coordinates');
    disp('  8 - Logarithmic and algebraic spirals');
    disp(' SCALAR AND VECTOR FIELDS');
    disp('  9 - Scalar field   f(x,y)=x^2-y^2');
    disp(' 10 - 2 scalar fields with same level curves');
    disp(' 11 - Vector field  F(x,y) = 2xi-2yj, quiver plot');
    disp(' 12 - Vector field  F(x,y) = 2xi-2yj, streamline plot ');
    disp(' 13 - Scalar field f=x^2-y^2 with its gradient');
    disp(' 14 - Graph surface of scalar field f=x^2-y^2');
    disp(' 15 - Scalar field evaluated on graph surface (for chain rule)');
    disp(' 16 - Graph surface of paraboloid and tangent plane');
    disp(' 17 - Representation of line integral of scalar fields');
    disp(' 18 - Vector field with positive divergence');
    disp(' 19 - Vector field with negative divergence');
    disp(' 20 - Vector field whose divergence changes sign');
    disp(' 21 - Vector field whose curl points into the page');
    disp(' 22 - Vector field whose curl normal component changes sign');
    disp(' 23 - 3 similar fields with positive/null/negative divergence');
    disp(' 24 - Vector fields in revision exercises of section 1');
    disp(' DOMAINS, CHANGES OF VARIABLES AND SURFACES');
    disp(' 25 - Unit square and 3 changes of variables');
    disp(' 26 - Smile domain and scalar field 1/y');
    disp(' 27 - Intersection of four parabolas');
    disp(' 28 - Pumpkin domain in spherical coordinates');
    disp(' 29 - Torus');
    disp(' 30 - Moebius strip');
    disp(' 31 - Quartic hump graph surface');
    disp(' 32 - Approximation of double integral with Riemann sum');
    choice = input('--->  ');
else			% if some input is given, take first input as choice1
    choice=varargin{1};
end

% exclude illegal choices and close function
if prod([1:32]-choice);  disp('Choice not allowed!');  return; end;

switch choice        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % CURVES
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    case 1
        %% curve 1:  helix
        np = 200;					% # evaluation points
        t0 = 0;						% curve extrema
        t1 = 6*pi;
        tt = linspace(t0,t1,np);	% build vector of equispaced "times"
        ax = cos(tt);				% evaluate three components of curve
        ay = sin(tt);
        az = 0.1*tt;
        plot3(ax,ay,az,'k','linewidth',2);			% make plot
        hold on; axis square;
        text(0,-1.4,0,'x'); text(-1.4,0,0,'y'); text(-1.2,1.2,1,'z');
        print -depsc2 Helix.eps
        
        %==================================================================
    case 2
        %% curve 2:  planar 3-leaved clover
        np = 200;
        t0 = 0;
        t1 = pi;
        tt = linspace(t0,t1,np);
        ax = cos(3*tt).*cos(tt);
        ay = cos(3*tt).*sin(tt);
        plot(ax,ay,'k','linewidth',2);						  % plot curve
        hold on;  axis equal;
        plot([0,0],[-1,1],'k--'); plot([xlim],[0,0],'k--');   % plot axis
        text(1.3,0.05,'x'); text(0.05,0.9,'y');
        print -depsc2 Clover.eps

        %==================================================================        
    case 3
        %% curve 3:  planar cubic
        np = 200;
        t0 = -0.99;
        t1 = 0.99;
        tt = linspace(t0,t1,np);
        ax = tt;
        ay = tt.^3;
        plot(ax,ay,'k','linewidth',2);
        hold on; axis equal;
        plot([0,0],[-1,1],'k--'); plot([xlim],[0,0],'k--');
        text(1.1,0.05,'x'); text(0.05,0.9,'y');
        print -depsc2 Cubic.eps
        % try to plot the spiral (choose a sufficiently long interval):
        %ax = cos(tt).*tt; ay = tt.*sin(tt); az = 0*tt;
        
        %==================================================================
    case 4
        %% Curves and their total derivatives in solutions of review exercises of section 1
        npt = 300;
        tmin = -1;
        tmax = 1;
        tt = linspace(tmin,tmax,npt);
        % define curves and their derivatives
        Ax = @(t) (t.^3-t);
        Ay = @(t) (1-t.^2);
        Bx = @(t) t.^3;
        By = @(t) t.^2;                  % singular in 0
        Cx = @(t) exp(t).*cos(2*pi*t);   % logarithmic spiral
        Cy = @(t) exp(t).*sin(2*pi*t);
        
        dAx = @(t) (3*t.^2-1);
        dAy = @(t) -2*t;
        dBx = @(t) 3*t.^2;
        dBy = @(t) 2*t;
        dCx = @(t) exp(t).*(cos(2*pi*t)-2*pi*sin(2*pi*t));
        dCy = @(t) exp(t).*(sin(2*pi*t)+2*pi*cos(2*pi*t));
        
        subplot(2,3,1)
        plot(Ax(tt),Ay(tt),'k','linewidth',2);
        hold on; axis square; title('a(t)');
        xl=xlim; yl=ylim;text(xl(2),yl(1),'x'); text(xl(1),yl(2)*1.1,'z');
        
        subplot(2,3,2)
        plot(Bx(tt),By(tt),'k','linewidth',2);
        hold on; axis square; title('b(t)');
        xl=xlim; yl=ylim;text(xl(2),yl(1),'x'); text(xl(1),yl(2)*1.1,'y');
        
        subplot(2,3,3)
        plot(Cx(tt),Cy(tt),'k','linewidth',2);
        hold on; axis square; title('c(t)');
        xl=xlim; yl=ylim;text(xl(2),yl(1),'x'); text(xl(1),yl(2)*1.1,'y');
        
        subplot(2,3,4)
        plot(dAx(tt),dAy(tt),'k','linewidth',2);
        hold on; axis square; title('da(t)dt');
        xl=xlim; yl=ylim;text(xl(2),yl(1),'x'); text(xl(1),yl(2)*1.1,'z');
        
        subplot(2,3,5)
        plot(dBx(tt),dBy(tt),'k','linewidth',2);
        hold on; axis square; title('db(t)dt');
        xl=xlim; yl=ylim;text(xl(2),yl(1),'x'); text(xl(1),yl(2)*1.1,'y');
        
        subplot(2,3,6)
        plot(dCx(tt),dCy(tt),'k','linewidth',2);
        hold on; axis square; title('dc(t)dt');
        xl=xlim; yl=ylim;text(xl(2),yl(1),'x'); text(xl(1),yl(2)*1.1,'y');
        
        print -depsc2 'RevExS1_Curves.eps'
        
        %==================================================================
    case 5
        %%  three different paths for line-integration of vector fields
        npv = 12;                           % set up for vector field quiver plot
        semiax = 2;
        xv = linspace(-0.1,1.1,npv);
        [xxv,yyv] = meshgrid(xv,xv);
        Vfx = 2*yyv;                        % evaluate vector field F =2yi-xj
        Vfy = -xxv;
        
        % quiver plot of the field:
        qplot = quiver(xxv,yyv,Vfx,Vfy);
        set(qplot,'color','k'); hold on; axis equal;
        % plot triangle:
        plot([0,1,1,0],[0,0,1,0],'k', 'linewidth', 2);
        % plot parabola (why is this the same path as t^2i+t^4j?):
        xxx = linspace(0,1,100);
        plot(xxx,xxx.^2,'k', 'linewidth', 2);
        % add some labels
        text(0.7,0.05,'G5');text(0.55,0.55,'G1');text(0.65,0.4,'G2');
        text(0,0.05,'u');text(0.95,1,'w'); text(0,1,'F');
        print -depsc2 'LineIntVector.eps'
        
        %==================================================================
    case 6
        %%   polar curves
        tt = linspace(-pi,pi,200);
        subplot(1,3,1);                             % propeller
        pplot = polar(tt,2+cos(3*tt));              % use Matlab's command polar
        set(pplot,'color','k', 'linewidth',2);
        subplot(3,3,2);                             % flowers
        pplot = polar(tt,2+cos(2*tt));
        set(pplot,'color','k', 'linewidth',2);
        subplot(3,3,5);
        pplot = polar(tt,2+cos(7*tt));
        set(pplot,'color','k', 'linewidth',2);
        subplot(3,3,8);
        pplot = polar(tt,2+cos(12*tt));
        set(pplot,'color','k', 'linewidth',2);
        
        ttt = linspace(0,2*pi,200);
        subplot(1,3,3);                             % two spirals
        pplot = polar(ttt,2*ttt);
        set(pplot,'color','k', 'linewidth',2);
        hold on;
        pplot = polar(ttt,ttt);
        set(pplot,'color','k', 'linewidth',2);
        pplot = polar([0,0],[2*pi,4*pi],'k');
        print -depsc2 'PolarPlots.eps'
        
        %==================================================================
    case 7
        %% 6 - curve in spherical coordinates, constraint to unit sphere
        t = linspace(0,pi,200);
        n = 10;
        splot = plot3(sin(t).*cos(n*t), sin(t).*sin(n*t), cos(t));
        set(splot,'color','k', 'linewidth',2); axis equal;
        print -depsc2 'SphericalCurve.eps'        
        
        %==================================================================
    case 8
        %% logarithmic and algebraic spirals
        np = 300;					% # evaluation points
        t1 = 30;                    % fix an arbitrary endpoint to approximate oo
        tt = linspace(0,t1,np); 	% build vector of equispaced "times"
        ttau = linspace(1,t1,np);
        ax = exp(-tt).*cos(tt);				% evaluate components of curve
        ay = exp(-tt).*sin(tt);
        bx = cos(ttau)./ttau;
        by = sin(ttau)./ttau;
        plot(ax,ay,'k','linewidth',2);
        hold on; axis equal;
        plot(bx,by,'k--','linewidth',2);
        plot([-1,1],[0,0],'k:');
        plot([0,0],[-.3,1],'k:');
        text(0.9,-.1,'i'); text(-.1,.95,'j');
        text(0.5,0.4,'a(t)'); text(-0.4,.5,'b(tau)');
        print -depsc2 '2Spirals.eps'
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % SCALAR AND VECTOR FIELDS
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
               
    case 9
        %% SCALAR FIELD   f(x,y)=x^2-y^2
        nps = 100;                              % # of plotting nodes in each direction
        semiax = 2;                             % domain is [-semiax,semiax]^2
        xs = linspace(-semiax,semiax,nps);      % build vector of nodes...
        [xxs,yys] = meshgrid(xs,xs);            % and transform it in two matrices!
        % (try to understand exactly what the meshgrid command does, it's crucial)
        
        Sf = xxs.^2 - yys.^2;                   % evaluate scalar field in nodes
        
        [C,h] = contour(xxs,yys,Sf, [-4:4],'linewidth', 2);   %contour plot
        clabel(C, h , 'fontsize',14);
        hold on; axis square;                   % ensure the plot keeps x-y ratio
        % plot Cartesian axes:
        plot([-semiax,semiax],[0,0],'k', [0,0],[-semiax,semiax],'k', 'linewidth', 2);
        text(2.1,0,'x'); text(0,2.1,'y');       % put axis labels
        print -depsc2 Sfield.eps
        
        %==================================================================
    case 10
        %% example of two scalar fields with the same level sets
        % f(r)=|r|^2  and  g(r)=exp{-|r|^2}
        nps = 100;                              % # of plotting nodes in each direction
        semiax = 2;                             % domain is [-semiax,semiax]^2
        xs = linspace(-semiax,semiax,nps);      % build vector of nodes...
        [xxs,yys] = meshgrid(xs,xs);            % and transform it in two matrices!
        
        Sff = xxs.^2 + yys.^2;                  % evaluate two fields in nodes
        SffG = exp(-xxs.^2 - yys.^2);
        
        subplot (1,2,1);                            % contour plot of f
        [C,h] = contour(xxs,yys,Sff, [0:7],'linewidth', 2);
        clabel(C, h , 'fontsize',14);
        hold on; axis square;
        plot([-semiax,semiax],[0,0],'k', [0,0],[-semiax,semiax],'k', 'linewidth', 2);
        text(2.1,0,'x'); text(0,2.2,'y');
        
        subplot (1,2,2);                            % contour plot of g
        % can try with uniformly-spaced level set:
        [C,h] = contour(xxs,yys,SffG, [0:0.1:1],'linewidth', 2);
        % or with the level set in the same positions of those for f
        %[C,h] = contour(xxs,yys,SffG, exp(-[0:10]),'linewidth', 2);
        clabel(C, h , 'fontsize',14);
        hold on; axis square;
        plot([-semiax,semiax],[0,0],'k', [0,0],[-semiax,semiax],'k', 'linewidth', 2);
        text(2.1,0,'x'); text(0,2.2,'y');
        print -depsc2 SfieldSpheres.eps
        
        % % you can also plot the two fields as surfaces to better understand them
        % subplot (1,2,3);
        % h = mesh(xxs,yys,Sff); hold on; axis square
        % text(0,-2.7,-4,'x');text(-2.7,0,-4,'y');text(-2.4,2.4,0,'z');
        % subplot (1,2,4);
        % h = mesh(xxs,yys,SffG); hold on; axis square;
        % text(0,-2.7,-4,'x');text(-2.7,0,-4,'y');text(-2.4,2.4,0,'z');
        
        %==================================================================
    case 11
        %%  VECTOR FIELD    F(x,y) = 2xi-2yj   quiver plot
        npv = 12;
        semiax = 2;
        xv = linspace(-semiax,semiax,npv);
        [xxv,yyv] = meshgrid(xv,xv);
        Vfx = 2*xxv;             % evaluate the 2 components of F in the mesh nodes
        Vfy = -2*yyv;
        
        quiver(xxv,yyv,Vfx,Vfy,'k','linewidth', 2); % quiver plot of F
        hold on; axis equal;
        plot(xlim,[0,0],'k', [0,0],[-semiax,semiax],'k', 'linewidth', 2);
        xl = xlim;
        text(xl(2)*1.05,0,'x'); text(0,2.1,'y');
        print -depsc2 Vfield.eps        
        
        %==================================================================
    case 12
        %%  VECTOR FIELD    F(x,y) = 2xi-2yj   streamline plot
        npv = 12;
        semiax = 2;
        xv = linspace(-semiax,semiax,npv);
        [xxv,yyv] = meshgrid(xv,xv);
        Vfx = 2*xxv;             % evaluate the 2 components of F in the mesh nodes
        Vfy = -2*yyv;
        
        h = streamslice(xxv,yyv,Vfx,Vfy);
        set(h,'color','k', 'linewidth',2);  hold on; axis square;
        plot([-semiax,semiax],[0,0],'k', [0,0],[-semiax,semiax],'k');
        text(2.1,0,'x'); text(0,2.1,'y');
        print -depsc2 VfieldLines.eps        
 
        %==================================================================
    case 13
        %% SCALAR FIELD f & VECTOR FIELD F TOGETHER
        % To see orthogonality of streamlines of F=grad f and level sets of f
        % First, prepare to draw level lines of f, requires fine grid: (see above)
        nps = 100;                              % # of plotting nodes in each direction
        semiax = 2;                             % domain is [-semiax,semiax]^2
        xs = linspace(-semiax,semiax,nps);      % build vector of nodes...
        [xxs,yys] = meshgrid(xs,xs);            % and transform it in two matrices
        Sf = xxs.^2 - yys.^2;                   % evaluate scalar field in nodes
        % Second, prepare to draw vector field on sparser grid:
        npv = 12;                               % # of nodes, now much lower
        semiax = 2;
        xv = linspace(-semiax,semiax,npv);
        [xxv,yyv] = meshgrid(xv,xv);
        Vfx = 2*xxv;             % evaluate the 2 components of F in the mesh nodes
        Vfy = -2*yyv;
        
        [C,h] = contour(xxs,yys,Sf, (-4:4),'linewidth', 3);
        clabel(C, h , 'fontsize',14); hold on; axis square;
        h = streamslice(xxv,yyv,Vfx,Vfy,0.5);
        set(h,'color','k');
        plot([-semiax,semiax],[0,0],'k', [0,0],[-semiax,semiax],'k', 'linewidth', 2);
        text(2.1,0,'x'); text(0,2.1,'y');
        print -depsc2 SVfields.eps
        
        %==================================================================
    case 14
        %% SCALAR FIELDS AS SURFACES  (f=x^2-y^2)
        % plot surface graph of f as a black mesh on [-2,2]^2
        np = 20;
        sa = 2;
        x = linspace(-sa,sa,np);
        [xx,yy] = meshgrid(x,x);
        zz = xx.^2-yy.^2;          % values of the field in the nodes
        
        h = mesh(xx,yy,zz);        % plot surface as a mesh
        colormap([0 0 0]);         % paint it black!
        hold on; axis square;
        text(0,-2.7,-4,'x'); text(-2.7,0,-4,'y'); text(-2.4,2.4,0,'z');
        print -depsc2 SurfX2mY2.eps
        
        %==================================================================
    case 15
        % now plot the same surface  z=x^2-y^2  on a finer mesh,
        % coloured with a special field   c(x,y,z) = x*sin(z)
        np = 150;
        sa = 2;
        x = linspace(-sa,sa,np);
        [xx,yy] = meshgrid(x,x);
        zz = xx.^2-yy.^2;             % evaluate value of f again on a finer mesh
        cc = xx.*sin(xx.^2-yy.^2);    % values of the colour field
        
        surf(xx,yy,zz,cc);            % surf(ace) plot, last entry cc sets the colour
        shading flat;                 % make it look nicer
        hold on; axis square;
        text(0,-2.7,-4,'x'); text(-2.7,0,-4,'y'); text(-2.4,2.4,0,'z');
        colorbar;                     % to decipher values of c from colours
        print -depsc2 SurfDTDP.eps
        
        %==================================================================
    case 16
        % plot surface graph of paraboloid and tangent plane
        np = 20;
        sa = 2;
        x = linspace(-sa,sa,np);
        [xx,yy] = meshgrid(x,x);
        zz = xx.^2+yy.^2;          % values of the field in the nodes
        
        xp = linspace(-1,1,np);
        yp = linspace(0,2,np);
        [xxp,yyp] = meshgrid(xp,yp);
        zzp = 2*yyp-1;             % values of the z component of tangent plane
        
        h = mesh(xx,yy,zz);        % plot surface as a mesh
        colormap([0 0 0]); hold on; axis square;
        text(0,-2.7,-4,'x'); text(-2.7,0,-4,'y'); text(-2.4,2.4,0,'z');
        mesh(xxp,yyp,zzp);
        view([1,2,0.5]);
        print -depsc2 SurfTangentPlane.eps
        
        %==================================================================
    case 17
        % now add planar curve and vertical surface to SurfX2mY2
        % used in Section 2 to explain line integrals
        np = 20;
        sa = 2;
        x = linspace(-sa,sa,np);
        [xx,yy] = meshgrid(x,x);
        zz = xx.^2-yy.^2+6;
        
        h(1) = mesh(xx,yy,zz);        % plot surface as a mesh
        hold on; axis square;
        npp = 50;
        tt = linspace(-2,2,npp);       % x-coordinate of planar curve
        ff = -1.5*exp(-tt.^2);         % y-coordinate of planar curve
        z = linspace(0,1,npp);         % build and plot vertical mesh
        [xx,zz] = meshgrid(tt,z);
        zz = zz .* (ones(npp,1)*(tt.^2-ff.^2+6));
        h(2) = surf(xx,ones(npp,1)*ff,zz);
        set(h(1), 'EdgeColor',[0 0 0]);
        set(h(2), 'FaceColor',[0.8 0.8 0.8],'EdgeColor',[0.6 0.6 0.6] );
        % plot planar curve and its graph:
        plot3(tt,ff,zeros(1,npp),'k','LineWidth',3);
        plot3(tt,ff,(tt.^2-ff.^2+6.05),'k','LineWidth',3);
        text(0,-2.7,0,'x'); text(-2.7,0,0,'y'); text(0,2,6,'f(x,y)');
        text(-1.2,-1,0,'a(t)');
        print -depsc2 SurfX2mY2withCurve.eps
                
        %==================================================================
    case 18
        %% Vector field F = x i + y j (identity field) with positive divergence
        npv = 12;
        semiax = 2;
        xv = linspace(-semiax,semiax,npv);
        [xxv,yyv] = meshgrid(xv,xv);			% set up mesh of points for evaluation
        Vfx1 = xxv;                             % evaluations of the field
        Vfy1 = yyv;
        quiver(xxv,yyv,Vfx1,Vfy1,'k','linewidth', 2);        		% quiver plot
        hold on; axis equal;
        plot(xlim,[0,0],'k', [0,0],ylim,'k', 'linewidth', 2);		% plot axes
        print -depsc2 PDiver.eps
        
        %==================================================================
    case 19
        %% Vector field F=(-x-2y)i+(2x-y)j with negative divergence
        % plotted on the same points as the previous one
        npv = 12;
        semiax = 2;
        xv = linspace(-semiax,semiax,npv);
        [xxv,yyv] = meshgrid(xv,xv);
        Vfx2 = -xxv-2*yyv;
        Vfy2 = 2*xxv-yyv;
        quiver(xxv,yyv,Vfx2,Vfy2,'k','linewidth', 2);
        hold on; axis equal;
        plot(xlim,[0,0],'k', [0,0],ylim,'k', 'linewidth', 2);
        print -depsc2 NDiver.eps
        
        %==================================================================
    case 20
        %% Vector field F=(x^2-x)i+(y^2-y)j  whose divergence changes
        npv = 12;
        ax = [-0.5, 1.5];
        xvv = linspace(ax(1), ax(2),npv);
        [xxvv,yyvv] = meshgrid(xvv,xvv);
        Vfx1 = xxvv.^2-xxvv;
        Vfy1 = yyvv.^2-yyvv;
        quiver(xxvv,yyvv,Vfx1,Vfy1,'k','linewidth', 2);
        hold on; axis equal;
        plot(xlim,[0,0],'k', [0,0],ylim,'k', 'linewidth', 2);
        % the line that divides the domain with div(F)>0 from that with div(F)<0
        plot([-0.5,1.5],[1.5,-0.5],'k--', 'linewidth', 2)
        print -depsc2 CSDiver.eps
        
        %==================================================================
    case 21
        %% Vector field F = 2yi-xj, whose curl "points into the page"
        % (evaluated on the same points of previous examples)
        npv = 12;
        semiax = 2;
        xv = linspace(-semiax,semiax,npv);
        [xxv,yyv] = meshgrid(xv,xv);
        VfxC = 2*yyv;
        VfyC = -xxv;
        quiver(xxv,yyv,VfxC,VfyC,'k','linewidth', 2);
        hold on; axis equal;
        plot(xlim,[0,0],'k', [0,0],ylim,'k', 'linewidth', 2);
        print -depsc2 Pcurl.eps
        
        %==================================================================
    case 22
        %% Vector field F = x^2 j, whose curl changes "sign" across the plane x=0
        npv = 12;
        semiax = 2;
        xv = linspace(-semiax,semiax,npv);
        [xxv,yyv] = meshgrid(xv,xv);
        VfxC = zeros(size(xxv));
        VfyC = xxv.^2;
        quiver(xxv,yyv,VfxC,VfyC,'k','linewidth', 2);
        hold on; axis equal;
        plot(xlim,[0,0],'k', [0,0],ylim,'k', 'linewidth', 2);
        print -depsc2 CScurl.eps
        
        %==================================================================
    case 23
        %% Confusing example!
        % three vector fields whose plots look almost the same
        % (and is positive-divergence looking)
        % but with positive / null / negative divergence!
        % they are (xi+yj)/(|x|^a)   with a = 1.8, 2 and 2.2, respectively
        % (note that here we are considering 2D fields)
        npv = 12;
        semiax = 0.5;
        xv = linspace(-semiax,semiax,npv);
        [xxv,yyv] = meshgrid(xv,xv);
        fx1 = xxv./(xxv.^2+yyv.^2).^0.9;   % evaluate 2 components for each of 3 fields
        fy1 = yyv./(xxv.^2+yyv.^2).^0.9;
        fx2 = xxv./(xxv.^2+yyv.^2).^1;
        fy2 = yyv./(xxv.^2+yyv.^2).^1;
        fx3 = xxv./(xxv.^2+yyv.^2).^1.1;
        fy3 = yyv./(xxv.^2+yyv.^2).^1.1;
        subplot (1,3,1); quiver(xxv,yyv,fx1,fy1,'k','linewidth', 2);
        hold on; axis square; plot(xlim,[0,0],'k', [0,0],ylim,'k', 'linewidth', 2);
        subplot (1,3,2); quiver(xxv,yyv,fx2,fy2,'k','linewidth', 2);
        hold on; axis square; plot(xlim,[0,0],'k', [0,0],ylim,'k', 'linewidth', 2);
        subplot (1,3,3); quiver(xxv,yyv,fx3,fy3,'k','linewidth', 2);
        hold on; axis square; plot(xlim,[0,0],'k', [0,0],ylim,'k', 'linewidth', 2);
        
        %==================================================================
    case 24
        %% quiver plots of vector fields in solution of revision ex of section 1
        np = 10;
        xmin = -1;   xmax = 1;     ymin = -1;     ymax = 1;
        xx = linspace(xmin,xmax,np);
        yy = linspace(ymin,ymax,np);
        [xm,ym] = meshgrid(xx,yy);
        % functions to define vector fields
        Fx = @(x,y) y; Fy = @(x,y) x;
        Gx = @(x,y) exp(-y.^2); Gy = @(x,y) 0*x;
        Hx = @(x,y) y; Hy = @(x,y) -x;
        Lx = @(x,y) x.*(x.^2+y.^2); Ly = @(x,y) y.*(x.^2+y.^2);
        Fxm = Fx(xm, ym);  Fym = Fy(xm, ym);
        Gxm = Gx(xm, ym);  Gym = Gy(xm, ym);
        Hxm = Hx(xm, ym);  Hym = Hy(xm, ym);
        Lxm = Lx(xm, ym);  Lym = Ly(xm, ym);
        
        subplot (2,2,1);   % this field is non planar
        %quiver(xm,ym,Fxm,Fym,'k','linewidth', 2);
        quiver3(xm,ym,0*xm,ym,xm,xm+ym,'k','linewidth', 1.5);
        hold on; axis square; title('F');
        streamslice(xm,ym,Fxm,Fym);
        xl=xlim; yl=ylim; zl=zlim;
        text(0,yl(1)*1.3,zl(1)*1.3,'x');
        text(xl(1)*1.3,0,zl(1)*1.3,'y');
        text(xl(1)*1.3,yl(2)*1.3,0,'z');
        
        subplot (2,2,2);
        quiver(xm,ym,Gxm,Gym,'k','linewidth', 1.5);
        hold on; axis square; title('G');
        streamslice(xm,ym,Gxm,Gym);
        xl=xlim; yl=ylim;text(xl(2),yl(1),'x'); text(xl(1),yl(2)*1.1,'z');
        
        subplot (2,2,3);
        quiver(xm,ym,Hxm,Hym,'k','linewidth', 1.5);
        hold on; axis square; title('H');
        streamslice(xm,ym,Hxm,Hym);
        xl=xlim; yl=ylim;text(xl(2),yl(1),'x'); text(xl(1),yl(2)*1.1,'y');
        
        subplot (2,2,4);
        quiver(xm,ym,Lxm,Lym,'k','linewidth', 1.5);
        hold on; axis square; title('L');
        streamslice(xm,ym,Lxm,Lym);
        xl=xlim; yl=ylim;text(xl(2),yl(1),'x'); text(xl(1),yl(2)*1.1,'y');
        
        print -depsc2 'RevExS1_Fields.eps'
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % DOMAINS, CHANGES OF VARIABLES AND SURFACES
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
    case 25
        %% plot unit square and 3 changes if variables, using a 5x5 mesh
        close all;
        nl = 4;                             % number of internal lines in each direction
        h = 1/(nl+1);                       % edge length of each little square
        segm = linspace(0,1,200);
        
        subplot(2,2,1);                     % plot the square
        plot([0,1,1,0,0],[0,0,1,1,0],'k', 'linewidth',3);
        axis square; hold on; %axis off;
        for j=1:nl
            plot([0,1],[h*j,h*j],'k');
            plot([h*j,h*j],[0,1],'k');
        end
        % this could be done also with:
        % xi1 = @(x,y) x; eta1 = @(x,y) y; PlotMyMesh(xi1,eta1,segm,h,nl);
        
        % For each shape we just have to define the parametrization xi and eta and
        % call the function PlotMyMesh (see at the end of this file)
        subplot(2,2,2);
        xi2 = @(x,y) x+0*y;
        eta2 = @(x,y) y+0.2*sin(2*pi*x);
        PlotMyMesh(xi2,eta2,segm,h,nl);
        
        subplot(2,2,3);
        xi3 = @(x,y) (1+x).*cos(y);
        eta3 = @(x,y) (1+x).*sin(y);
        PlotMyMesh(xi3,eta3,segm,h,nl);
        
        subplot(2,2,4);
        xi4 = @(x,y) exp(x*y).*cos(2*pi*y);
        eta4 = @(x,y) exp(x*y).*sin(2*pi*y);
        PlotMyMesh(xi4,eta4,segm,h,nl);
        
        print -depsc2 'ChangeVariables.eps'
     
        %==================================================================
    case 26
        %% smile domain and scalar field 1/y
        nx = 20;        % # points in x and y directions
        ny = 10;
        xi = linspace(-1,1,nx);
        eta = linspace(0,1,ny);
        [xi_mg,eta_mg] = meshgrid(xi,eta);
        
        a = @(x) 2*x.^2-2;          % functions defining domain boundary
        b = @(x) x.^2-1;
        
        x = @(xi,eta) xi;           % change of variables
        y = @(xi,eta) (b(xi)-a(xi)).*eta+a(xi);
        f = @(x,y) 1./y;            % integrand
        
        x_mg = x(xi_mg,eta_mg);     % apply change of variables in nodes
        y_mg = y(xi_mg,eta_mg);
        f_mg = f(x_mg,y_mg);        % evaluate integrand
        
        subplot(1,2,1);             % domain
        mesh(x_mg,y_mg,0*x_mg);
        axis equal; hold on;view([0,0,1]);text(1.1,-2,'x');text(-1.2,0,'y');
        subplot(1,2,2);             % field to be integrated
        mesh(x_mg,y_mg,f_mg);
        axis equal; hold on;colormap([0 0 0]); view([-3,-1,5]);
        print -depsc2 'SmileDomain.eps'
        
        %==================================================================
    case 27
        %% intersection of four parabolas
        segm = linspace(0.15,1.2,500);
        plot(segm, sqrt(segm), 'k', 'linewidth',2);
        hold on; axis square;
        plot(sqrt(segm), segm, 'k', 'linewidth',2);
        plot(segm, sqrt(segm/3), 'k', 'linewidth',2);
        plot(sqrt(segm/2), segm, 'k', 'linewidth',2);
        text(0.9,0.8,'r');
        text(0.4,1,'l');
        text(0.9,0.5,'b');
        text(0.3,0.65,'t');
        text(0.7,0.65,'R');
        % (labels seem crazily placed but will work fine in the final file)
        print -depsc2 'FourParabolas.eps'
        
        %==================================================================
    case 28
        %% pumpkin domain in spherical coordinates
        ns = 100;
        phi = linspace (0,pi,ns);
        theta = linspace (-pi,pi,ns);
        [phi_mg,theta_mg] = meshgrid(phi,theta);    % build an equispaced mesh in phi and theta
        % surface is defined by a function (radius) of phi and theta:
        rho = @(phi,theta) (sin(phi)) .* (5+cos(7*theta));
        rho_mg = rho(phi_mg,theta_mg);
        x_mg = rho_mg.*sin(phi_mg).*cos(theta_mg);  % convert rho/theta/phi in Cartesian coordinates
        y_mg = rho_mg.*sin(phi_mg).*sin(theta_mg);
        z_mg = rho_mg.*cos(phi_mg);
        mesh(x_mg,y_mg,z_mg); colormap([0 0 0]);  axis equal;
        print -depsc2 'SphericalPumpkin.eps'
        
        %==================================================================
    case 29
        %% Draw a torus
        a = 3;    % radii a and b, b<a
        b = 1;
        np = 20;
        u = linspace(-pi,pi,np);
        w = linspace(-pi,pi,np);
        [uu,ww] = meshgrid(u,w);
        X1 = (a+b*cos(ww)).*cos(uu);
        X2 = (a+b*cos(ww)).*sin(uu);
        X3 = b*sin(ww);
        
        mesh(X1,X2,X3);
        hold on; colormap([0 0 0]);  axis equal;
        % draw 2 circles corresponding to mapped boundary of region (-pi,pi)^2
        tt =linspace(-pi,pi,100);
        plot3( -(a+b*cos(tt)), 0*tt, b*sin(tt), 'r', 'LineWidth', 3);
        plot3( (a-b-0.05)*cos(tt), (a-b-0.05)*sin(tt), 0*tt, 'b', 'LineWidth', 3);
        print -depsc2 'Torus.eps'
        
        %==================================================================
    case 30
        %% Draw a Moebius strip
        npu = 40;
        npw = 15;
        u = linspace(0,2*pi,npu);
        w = linspace(-0.5,0.5,npw);
        [uu,ww] = meshgrid(u,w);
        X1 = (1 + ww.*cos(uu/2)).*cos(uu);
        X2 = (1 + ww.*cos(uu/2)).*sin(uu);
        X3 = ww.*sin(uu/2);
        
        mesh(X1,X2,X3);
        hold on; colormap([0 0 0]);  axis equal; view([1,-1,2]);
        % draw 2 lines corresponding to mapped boundary of region (-pi,pi)^2
        ttw = linspace(-0.5,0.5,100);
        plot3( (1 + ttw), 0*ttw, 0*ttw, 'r', 'LineWidth', 3);
        ttu = linspace(0,2*pi,100);
        plot3( (1 + 0.5*cos(ttu/2)).*cos(ttu), (1 + 0.5.*cos(ttu/2)).*sin(ttu), 0.5*sin(ttu/2), 'b', 'LineWidth', 3);
        plot3( (1 - 0.5*cos(ttu/2)).*cos(ttu), (1 - 0.5.*cos(ttu/2)).*sin(ttu), -0.5*sin(ttu/2), 'b', 'LineWidth', 3);
        print -depsc2 'Moebius.eps'
        
        %==================================================================
    case 31
        %% Draw a quartic hump surface
        %VCplotter(2,@(x,y) x.*(1-x).*y.*(1-y),0,1,0,1);
        np = 20;
        x = linspace(0,1,np);
        [xx,yy] = meshgrid(x,x);
        zz = 16*xx.*(1-xx).*yy.*(1-yy);          % values of the field in the nodes
        mesh(xx,yy,zz);        % plot surface as a mesh
        colormap([0 0 0]); hold on; axis equal;
        % draw unit normal:
        plot3([1/2,1/2],[1/4,1/4-0.4], [3/4,3/4+0.2],'k','LineWidth',2);
        plot3(1/2,1/4-0.4, 3/4+0.2,'k>','LineWidth',2);
        % draw tangent vectors using quiver3: (1.5 is scaling factor for plotting)
        dgdx = 16*(1-2*xx).*(yy-yy.^2);
        dgdy = 16*(1-2*yy).*(xx-xx.^2);
        quiver3(xx,yy,zz,ones(size(xx)),zeros(size(xx)),dgdx,1.5, 'r','LineWidth',2);
        quiver3(xx,yy,zz,zeros(size(xx)),ones(size(xx)),dgdy,1.5, 'b','LineWidth',2);
        print -depsc2 'Hump.eps'

        %==================================================================
    case 32
        %% Approximation of double integral with Riemann sum on rectangle
        nx = 5; ny = 5;     % number of rectangles in x and y directions
        x0 = 0; x1 = 1;     % boundary of rectangular region
        y0 = 0; y1 = 1;
        f = @(x,y) sqrt(x+y/2);     % (positive) scalar field of which we approximate integral
        
        hx = (x1-x0)/nx;            % meshsize in x and y direction
        hy = (y1-y0)/ny;
        x = linspace(x0,x1,nx+1);   % mesh nodes
        y = linspace(y0,y1,ny+1);
        [xx,yy] = meshgrid(x,y);
        ff = f(xx,yy);
        RiemannSum = 0;             % Compute approximation of integral from below
        
        figure; hold on;
        for i = 1:nx                % loop over all small rectangles
            for j = 1:ny 
                % choose minimum of f on 4 vertices to ensure approximation lies under exact field (note indices i,j swapped)
                fmin = min([ff(j,i),ff(j,i+1),ff(j+1,i),ff(j+1,i+1)]);     
                patch([x(i),x(i+1),x(i+1),x(i)],    [y(j),y(j),y(j+1),y(j+1)],      fmin*[1,1,1,1],fmin); % draw upper face
                patch([x(i),x(i+1),x(i+1),x(i)],    [y(j),y(j),y(j),y(j)],          fmin*[1,1,0,0],fmin); % draw 4 side faces
                patch([x(i),x(i+1),x(i+1),x(i)],    [y(j+1),y(j+1),y(j+1),y(j+1)],  fmin*[1,1,0,0],fmin); 
                patch([x(i),x(i),x(i),x(i)],        [y(j),y(j+1),y(j+1),y(j)],      fmin*[1,1,0,0],fmin); 
                patch([x(i+1),x(i+1),x(i+1),x(i+1)],[y(j),y(j+1),y(j+1),y(j)],      fmin*[1,1,0,0],fmin); 
                RiemannSum = RiemannSum + hx*hy*fmin;
            end
        end
        view([0.5,-1,1]);
        % draw finer mesh to represent exact field
        nnx = 4*nx+1;
        nny = 4*ny+1;
        xxx = linspace(x0,x1,nnx);
        yyy = linspace(y0,y1,nny);
        [xxxx,yyyy] = meshgrid(xxx,yyy);
        ffff = f(xxxx,yyyy);
        mesh(xxxx,yyyy,ffff,'FaceAlpha',0,'EdgeColor',[0,0,0],'Linewidth',1);
        print -depsc2 'Riemann2D.eps'
        sprintf('Value of Riemann sum is %d', RiemannSum)
     
        % Second figure, double integral as volume to be approximated
        figure
        surf(xxxx,yyyy,f(xxxx,yyyy));        % surface
        view([0.5,-1,1]);
        patch([x1,x0,xxx],y0*ones(1,nnx+2),[0,0,ffff(1,:)],1); % add 4 sides
        patch([x1,x0,xxx],y1*ones(1,nnx+2),[0,0,ffff(end,:)],1);
        patch(x0*ones(1,nny+2),[y1,y0,yyy],[0,0,ffff(:,1)'],1);
        patch(x1*ones(1,nny+2),[y1,y0,yyy],[0,0,ffff(:,end)'],1);
        print -depsc2 'DoubleIntegralVolume.eps'
        
end; % close switch
bye = sprintf('Choice %d, plot done. Bye!', choice);
disp(bye)
end % close function


%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%% auxiliary function that does plot for change of variables
function PlotMyMesh(xi,eta,segm,h,nl)
plot(xi(segm,0), eta(segm,0),'k', 'linewidth',3);
axis equal; hold on;
plot(xi(0,segm), eta(0,segm),'k', 'linewidth',3);
plot(xi(segm,1), eta(segm,1),'k', 'linewidth',3);
plot(xi(1,segm), eta(1,segm),'k', 'linewidth',3);
for j=1:nl
    plot(xi(j*h,segm), eta(j*h,segm),'k');
    plot(xi(segm,j*h), eta(segm,j*h),'k');
end
xl=xlim; yl=ylim;text(xl(2), yl(1),'xi');text(xl(1), yl(2),'eta');
end