% VCplotter.m       MA2VC-MA3VC - University of Reading    Andrea Moiola
% October 2013, updated in September 2014 and 2015
%
% Plot various geometrical entities studied in class:
% curves, scalar fields, vector fields, domains, surfaces
% in different coordinates systems.
%
% You can either:
% _ 1 _ just type  VCplotter  
%       and type the necessary data when required by the interface;
% _ 2 _ call the function with arguments, e.g.:
%%%% CURVES:
% VCplotter(1,1,@(t) (1+t)*cos(t),@(t) (1+t)*sin(t),0,2*pi);     % planar Cartesian curve (Archimedean spiral)
% VCplotter(1,2,@(t) t^2,@(t) sin(t),0,20);                      % planar polar curve
% VCplotter(1,3,@(t) t,@(t) t*sin(1/t),@(t) cos(10*t)*t^2,0,1);  % 3D cartesian curve
% VCplotter(1,4,@(t)t,@(t)t,@(t)t,0,100);                        % 3D cylindrical curve (cone)
% VCplotter(1,5,@(t)1,@(t)t,@(t)t,0,2*pi);                       % 3D spherical curve    
%%%% FIELDS:
% VCplotter(2,@(x,y) x^2+cos(5*y),-1,1,0,3);                     % 2D scalar Cartesian field
% VCplotter(3,@(r,theta) cos(5*theta)*r^2/(1+r^2),2);            % 2D scalar polar field
% VCplotter(4,@(x,y) -x-y,@(x,y) x-y,-1,1,-1,1);                 % 2D vector Cartesian field
% VCplotter(5,@(r,theta) 0,@(r,theta) cos(3*theta)*(r^2-r),1);   % 2D vector polar field
%%%% DOMAINS:
% VCplotter(6,@(x,y) x-y,@(x,y) y+sin(x),-pi,pi,-2,2);           % 2D domain mapping
% VCplotter(7,@(theta,z) (1-z^2)*(2+sin(6*theta)),-1,1);         % cylindrical domain
% VCplotter(8,@(phi,theta) 2+sin(3*phi)+cos(5*theta));           % spherical domain
%%%% SURFACES:
% VCplotter(9, @(u,w)(3+cos(w))*cos(u),  @(u,w)(3+cos(w))*sin(u),  @(u,w) sin(w), -pi, pi, -pi, pi);    % torus
% VCplotter(9, @(u,w)(1+w*cos(u/2))*cos(u),  @(u,w)(1+w*cos(u/2))*sin(u),  @(u,w) w*sin(u/2), 0, 2*pi, -0.5, 0.5);    % Moebius


function PlotHandle = VCplotter(varargin)
format compact; close all;
disp('Welcome to VCplotter.m!');

%% choice 1
if nargin==0    % if no input is given, ask:
    disp('What do you want to plot?  Choose your option:');
    disp('  1 - Curve');
    disp('  2 - 2D scalar field in Cartesian coordinates');
    disp('  3 - 2D scalar field in polar coordinates');
    disp('  4 - 2D vector fields in Cartesian coordinates');
    disp('  5 - 2D vector fields in polar coordinates');
    disp('  6 - Mapping of 2D domains');
    disp('  7 - Domains in cylindrical coordinates');
    disp('  8 - Domains in spherical coordinates');
    disp('  9 - Parametric surface (on rectangular region)');
    choice1 = input('--->  ');
else			% if some input is given, take first input as choice1
    choice1=varargin{1};
end

% exclude illegal choices and close function
if prod([1:9]-choice1);  disp('Choice not allowed!');  return; end;

switch choice1
    case 1 % ====== CURVES ======
        nc = 1000;   % number of points plotted
        %% choice 2 for curves
        if nargin<2  % if no second input is given
            disp('Which kind of curve?  Choose your option:');
            disp('  1 - planar curve in Cartesian coordinates');
            disp('  2 - planar curve in polar coordinates');
            disp('  3 - 3D curve in Cartesian coordinates');
            disp('  4 - 3D curve in cylindrical coordinates');
            disp('  5 - 3D curve in spherical coordinates');
            choiceCurve = input('--->  ');
        else        % if a second input is given
            choiceCurve=varargin{2};
        end
        % exclude illegal choices and close function
        if prod([1,2,3,4,5]-choiceCurve); disp('Choice not allowed!'); return; end;
        % switch between possible curves
        switch choiceCurve
            case 1 % planar Cartesian curve, 
                % (cases 1/2)accept input only if the number of arguments 
                % is 6 (at least, ignore others):  
                % 2 choices, 2 components, initial/final times
                if nargin>=6
                    xc = varargin{3}; yc = varargin{4};
                    tI = varargin{5}; tF = varargin{6};
                else
                    disp('Type component x of your curve, e.g.:  @(t) (1+t)*cos(t)'); xc = input('--->  ');
                    disp('Type component y of your curve, e.g.:  @(t) (1+t)*sin(t)'); yc = input('--->  ');
                    disp('Type initial time, e.g.:  0');  tI = input('--->  ');
                    disp('Type final time, e.g.:  2*pi'); tF = input('--->  ');
                end
                figure; tt =linspace(tI,tF,nc); xx = zeros(1,nc); yy = xx;
                % in the code there are a lot of "for" loops, which make it
                % slow and inefficient, they are necessary to allow
                % non-vectorised function inputs 
                % (eg, it takes (1+t)*cos(t) as (1+t).*cos(t))
                for j=1:nc; xx(j) = xc(tt(j)); yy(j) = yc(tt(j)); end;
                PlotHandle = plot(xx,yy,'k','linewidth',2);  %axis equal;
                set (gca (), 'dataaspectratio', [1 1 1]); 
                
            case 2 % planar Polar curve, 
                if nargin>=6
                    rc = varargin{3}; thetac = varargin{4};
                    tI = varargin{5}; tF = varargin{6};
                else
                    disp('Type radial component of your curve, e.g.:  @(t) t^2');       rc = input('--->  ');
                    disp('Type angular component of your curve, e.g.:  @(t) sin(t)');  thetac = input('--->  ');
                    disp('Type initial time, e.g.:  0');  tI = input('--->  ');
                    disp('Type final time, e.g.:  20');   tF = input('--->  ');
                end
                figure; tt =linspace(tI,tF,nc); rr = zeros(1,nc); ttheta = rr;
                for j=1:nc; rr(j) = rc(tt(j)); ttheta(j) = thetac(tt(j)); end;
                PlotHandle = polar(ttheta,rr,'k');
                
            case 3 % 3D Cartesian curve, 
                % (cases 3/4/5)accept input only if the number of arguments 
                % is 7 (at least, ignore others):  
                % 2 choices, 3 components, initial/final times
                if nargin>=7
                    xc = varargin{3}; yc = varargin{4}; zc = varargin{5};
                    tI = varargin{6}; tF = varargin{7};
                else
                    disp('Type component x of your curve, e.g.:  @(t) t'); xc = input('--->  ');
                    disp('Type component y of your curve, e.g.:  @(t) t*sin(1/t)'); yc = input('--->  ');
                    disp('Type component z of your curve, e.g.:  @(t) cos(10*t)*t^2'); zc = input('--->  ');
                    disp('Type initial time, e.g.:  0');  tI = input('--->  ');
                    disp('Type final time, e.g.:  1'); tF = input('--->  ');
                end
                figure; tt =linspace(tI,tF,nc); xx = zeros(1,nc); yy = xx; zz = xx;
                for j=1:nc; xx(j) = xc(tt(j)); yy(j) = yc(tt(j)); zz(j) = zc(tt(j)); end;
                PlotHandle = plot3(xx,yy,zz,'k'); %axis equal;
                set (gca (), 'dataaspectratio', [1 1 1]); 
                
            case 4 % 3D cylindrical curve, 
                if nargin>=7
                    rc = varargin{3}; thetac = varargin{4}; zc = varargin{5};
                    tI = varargin{6}; tF = varargin{7};
                else
                    disp('Type radial component (r) of your curve, e.g.:  @(t) t');      rc = input('--->  ');
                    disp('Type angular component (theta) of your curve, e.g.:  @(t) t'); thetac = input('--->  ');
                    disp('Type z component of your curve, e.g.:  @(t) t');               zc = input('--->  ');
                    disp('Type initial time, e.g.:  0');  tI = input('--->  ');
                    disp('Type final time, e.g.:  100'); tF = input('--->  ');
                end
                figure; tt =linspace(tI,tF,nc); rr = zeros(1,nc); ttheta = rr; zz = rr;
                for j=1:nc; rr(j) = rc(tt(j)); ttheta(j) = thetac(tt(j)); zz(j) = zc(tt(j)); end;
                PlotHandle = plot3(rr.*cos(ttheta),rr.*sin(ttheta),zz,'k'); %axis equal;                
                set (gca (), 'dataaspectratio', [1 1 1]); 
                
            case 5 % 3D spherical curve, 
                if nargin>=7
                    rc = varargin{3}; phic = varargin{4}; thetac = varargin{5};
                    tI = varargin{6}; tF = varargin{7};
                else
                    disp('Type radial component (rho) of your curve, e.g.:  @(t) 1');  rc = input('--->  ');
                    disp('Type colatitude component (phi) of your curve, e.g.:  @(t) t'); phic = input('--->  ');
                    disp('Type longitudinal component (theta) of your curve, e.g.:  @(t) t');  thetac = input('--->  ');
                    disp('Type initial time, e.g.:  0'); tI = input('--->  ');
                    disp('Type final time, e.g.:  2*pi');  tF = input('--->  ');
                end
                figure; tt =linspace(tI,tF,nc); rr = zeros(1,nc); pphi = rr; ttheta = rr; 
                for j=1:nc; rr(j) = rc(tt(j)); pphi(j) = phic(tt(j)); ttheta(j) = thetac(tt(j)); end;
                PlotHandle = plot3(rr.*sin(pphi).*cos(ttheta),rr.*sin(pphi).*sin(ttheta),rr.*cos(pphi),'k'); %axis equal;
                set (gca (), 'dataaspectratio', [1 1 1]);  end
        % end of curves
                
    case 2 % ====== 2D SCALAR PLOT - Cartesian ======
        % accept input only if the number of arguments is 6 (at least, ignore others):  
        % 1 choice, 1 field, 4 domain bounds
        if nargin>=6
            f = varargin{2}; x1 = varargin{3}; x2 = varargin{4}; y1 = varargin{5}; y2 = varargin{6};
        else
            disp('Type field f(x,y), e.g.:  @(x,y) x^2+cos(5*y)');  f = input('--->  ');
            disp('Type left bound of your rectangular domain, e.g.:  -1');  x1 = input('--->  ');
            disp('Type right bound of your rectangular domain, e.g.:  1');  x2 = input('--->  ');
            disp('Type lower bound of your rectangular domain, e.g.:  0');  y1 = input('--->  ');
            disp('Type upper bound of your rectangular domain, e.g.:  3');  y2 = input('--->  ');
        end
        nf = 100; x =linspace(x1,x2,nf); y = linspace(y1,y2,nf); [xg,yg] = meshgrid(x,y); fg = xg;
        % (in general, variable with names ending in -g will be values on a grid, from meshgrid)
        for j=1:nf; for k=1:nf; fg(j,k) = f(xg(j,k),yg(j,k)); end; end;
        figure;  % level lines and surface:
        subplot(1,2,1); [C,PlotHandle] = contour(xg,yg,fg,'linewidth', 2); clabel(C,PlotHandle,'fontsize',14); axis equal;       
        subplot(1,2,2); surf(xg,yg,fg); shading flat; % axis equal; 
        set (gca (), 'dataaspectratio', [1 1 1]); 

    case 3 % ====== 2D SCALAR PLOT - polar ======
        % accept input only if the number of arguments is 3 (at least, ignore others):  
        % 1 choice, 1 field, 1 maximal domain radius
        if nargin>=3
            f = varargin{2}; R = varargin{3}; 
        else
            disp('Type field f(r,theta), e.g.:  @(r,theta) cos(5*theta)*r^2/(1+r^2)'); f = input('--->  ');
            disp('Type maximal radius of your domain, e.g.:  2');  R = input('--->  ');
        end
        nf = 100; r =linspace(eps,R,nf); theta = linspace(-pi,pi,nf); [rg,thetag] = meshgrid(r,theta); fg = rg;
	    for j=1:nf; for k=1:nf; fg(j,k) = f(rg(j,k), thetag(j,k)); end; end;
        figure;  % level lines and surface:
        subplot(1,2,1); [C,PlotHandle] = contour(rg.*cos(thetag),rg.*sin(thetag),fg,'linewidth', 2); 
        axis equal;  clabel(C,PlotHandle,'fontsize',14);      
        subplot(1,2,2); surf(rg.*cos(thetag),rg.*sin(thetag),fg); shading flat; % axis equal;
        set (gca (), 'dataaspectratio', [1 1 1]); 
        
    case 4  % ====== 2D VECTOR PLOT - Cartesian ====== 
        % accept input only if the number of arguments is 7 (at least, ignore others):  
        % 1 choice, 2 fields, 4 domain bounds
        if nargin>=7
            Fx = varargin{2}; Fy = varargin{3}; x1 = varargin{4}; x2 = varargin{5}; y1 = varargin{6}; y2 = varargin{7};
        else
            disp('Type first component of field F1(x,y), e.g.:  @(x,y) -x-y'); Fx = input('--->  ');
            disp('Type second component of field F2(x,y), e.g.:  @(x,y) x-y'); Fy = input('--->  ');
            disp('Type left bound of your rectangular domain, e.g.:  -1');  x1 = input('--->  ');
            disp('Type right bound of your rectangular domain, e.g.:  1');  x2 = input('--->  ');
            disp('Type lower bound of your rectangular domain, e.g.:  -1'); y1 = input('--->  ');
            disp('Type upper bound of your rectangular domain, e.g.:  1');  y2 = input('--->  ');
        end
		nF = 12; x = linspace(x1,x2,nF); y = linspace(y1,y2,nF);  [xg,yg] = meshgrid(x,y);
	    for j=1:nF; for k=1:nF; Fxg(j,k) = Fx(xg(j,k), yg(j,k)); Fyg(j,k) = Fy(xg(j,k), yg(j,k)); end; end;
        figure;  % quiver plot, no streamline since not available in Octave
        PlotHandle = quiver(xg,yg,Fxg,Fyg); set(PlotHandle,'color','black','linewidth', 2); %axis equal;  
        set (gca (), 'dataaspectratio', [1 1 1]);      

    case 5  % ====== 2D VECTOR PLOT - Polar ====== 
        % accept input only if the number of arguments is 4 (at least, ignore others):  
        % 1 choice, 2 fields, 1 maximal radius
        if nargin>=4
            Fr = varargin{2}; Ftheta = varargin{3}; R = varargin{4};
        else
            disp('Type radial component of field Fr(r,theta), e.g.:  @(r,theta) 0'); Fr = input('--->  ');
            disp('Type angular component of field Ftheta(r,theta), e.g.:  @(r,theta) cos(3*theta)*(r^2-r)'); Ftheta = input('--->  ');
            disp('Type maximal radius of your domain, e.g.:  1');  R = input('--->  ');
        end
		nF = 20; r = linspace(eps,R,nF); theta = linspace(-pi,pi,nF);  [rg,thetag] = meshgrid(r,theta);
	    for j=1:nF; for k=1:nF; Frg(j,k) = Fr(rg(j,k), thetag(j,k)); Fthetag(j,k) = Ftheta(rg(j,k), thetag(j,k)); end; end;
        figure;  % quiver plot, no streamline since not available in Octave
		% now need to convert both input and output to Cartesian coordinates to use quiver
		PlotHandle = quiver(rg.*cos(thetag),rg.*sin(thetag),cos(thetag).*Frg-sin(thetag).*Fthetag,sin(thetag).*Frg+cos(thetag).*Fthetag);
		set(PlotHandle,'color','black','linewidth', 2); %axis equal;     
        set (gca (), 'dataaspectratio', [1 1 1]);   
		
    case 6 % ====== MAPPING OF A RECTANGLE - Cartesian ====== 
        % accept input only if the number of arguments is 7 (at least, ignore others):  
        % 1 choice, 2 fields, 4 domain bounds
        if nargin>=7
            xi = varargin{2}; eta = varargin{3}; x1 = varargin{4}; x2 = varargin{5}; y1 = varargin{6}; y2 = varargin{7};
        else
            disp('Type first component of change of variables xi(x,y), e.g.:  @(x,y) x-y'); xi = input('--->  ');
            disp('Type second component of change of variables eta(x,y), e.g.:  @(x,y) y+sin(x)'); eta = input('--->  ');
            disp('Type left bound of your rectangular domain, e.g.:  -pi');  x1 = input('--->  ');
            disp('Type right bound of your rectangular domain, e.g.:  pi');  x2 = input('--->  ');
            disp('Type lower bound of your rectangular domain, e.g.:  -2'); y1 = input('--->  ');
            disp('Type upper bound of your rectangular domain, e.g.:  2');  y2 = input('--->  ');
        end
		nD = 500; nl = 4;  % nD = points along edges for plot, nl = number of subdivisions in plotted mesh
		x = linspace(x1,x2,nD); y = linspace(y1,y2,nD);  
		% x and y are vectors of coordinates along edges of rectangle in xy-plane, 
        hx = (x2-x1)/(nl+1); hy = (y2-y1)/(nl+1); vx = 0*x; vy = vx;
        % hx/hy meshsize, vx/vy temporary vectors with values for plots
		PlotHandle = figure; 
		for k=1:nD; vx(k) = xi(x(k),y1); vy(k) = eta(x(k),y1); end; plot(vx,vy,'k', 'linewidth',3); % lower edge
		hold on;
		for k=1:nD; vx(k) = xi(x(k),y2); vy(k) = eta(x(k),y2); end; plot(vx,vy,'k', 'linewidth',3); % upper edge
		for k=1:nD; vx(k) = xi(x1,y(k)); vy(k) = eta(x1,y(k)); end; plot(vx,vy,'k', 'linewidth',3); % left edge
		for k=1:nD; vx(k) = xi(x2,y(k)); vy(k) = eta(x2,y(k)); end; plot(vx,vy,'k', 'linewidth',3); % right edge
        set (gca (), 'dataaspectratio', [1 1 1]); % axis equal;        
		for j=1:nl; for k=1:nD; vx(k) = xi(x(k),y1+j*hy); vy(k) = eta(x(k),y1+j*hy); end; plot(vx,vy,'k', 'linewidth',3); % horizontal edge
			for k=1:nD; vx(k) = xi(x1+j*hx,y(k)); vy(k) = eta(x1+j*hx,y(k)); end; plot(vx,vy,'k', 'linewidth',3); % horizontal edge
		end
		
    case 7 % ====== Cylindrical domain, in form r<f(theta,z) ====== 
        % accept input only if the number of arguments is 4 (at least, ignore others):  
        % 1 choice, 1 function, 2 extrema for z
       if nargin>=4
            f = varargin{2}; z1 = varargin{3}; z2 = varargin{4};
        else
            disp('Type cylindrical function defining the domain, e.g.:  @(theta,z) (1-z^2)*(2+sin(6*theta))'); f = input('--->  ');
            disp('Type lower bound for z, e.g.:  -1');  z1 = input('--->  ');
            disp('Type upper bound for z, e.g.:  1');  z2 = input('--->  ');
		end
		nD = 100; theta = linspace(-pi,pi,nD); z = linspace(z1,z2,nD); [thetag,zg] = meshgrid(theta,z); rg = 0*zg;
		for j = 1:nD; for k=1:nD; rg(j,k)=f(thetag(j,k),zg(j,k)); end; end;
		figure; PlotHandle = surf(rg.*cos(thetag),rg.*sin(thetag),zg); %shading flat; 
        set (gca (), 'dataaspectratio', [1 1 1]); %axis equal; 
        
    case 8 % ====== Spherical domain, in form rho<f(phi,theta) ====== 
        % accept input only if the number of arguments is 2 (at least, ignore others):  
        % 1 choice, 1 function
       if nargin>=2;
            f = varargin{2}; 
        else
            disp('Type spherical function defining the domain, e.g.:  @(phi,theta) 2+sin(3*phi)+cos(5*theta)'); f = input('--->  ');
		end
		nD = 100; phi= linspace(0,pi,nD); theta = linspace(-pi,pi,nD); [phig,thetag] = meshgrid(phi,theta); rhog = 0*phig;
		for j = 1:nD; for k=1:nD; rhog(j,k)=f(phig(j,k),thetag(j,k)); end; end;
		figure; PlotHandle = surf(rhog.*sin(phig).*cos(thetag),rhog.*sin(phig).*sin(thetag),rhog.*cos(phig)); 
        set (gca (), 'dataaspectratio', [1 1 1]); % axis equal;    
    
    case 9 % ====== parametric surface, in form X(u,w) ====== 
        % accept input only if the number of arguments is 9 (at least, ignore others):  
        % 1 choice, 2 fields, 4 domain bounds
        if nargin>=8
            X1 = varargin{2}; X2 = varargin{3}; X3 = varargin{4}; u1 = varargin{5}; u2 = varargin{6}; w1 = varargin{7}; w2 = varargin{8};
        else
            disp('Type first component of chart X1(u,w), e.g.:  @(u,w) (3+cos(w))*cos(u)'); X1 = input('--->  ');
            disp('Type second component of chart X2(u,w), e.g.:  @(u,w) (3+cos(w))*sin(u)'); X2 = input('--->  ');
            disp('Type third component of chart X3(u,w), e.g.:  @(u,w) sin(w)'); X3 = input('--->  ');
            disp('Type left  bound of rectangular region, e.g.:  -pi');  u1 = input('--->  ');
            disp('Type right bound of rectangular domain, e.g.:   pi');  u2 = input('--->  ');
            disp('Type lower bound of rectangular domain, e.g.:  -pi');  w1 = input('--->  ');
            disp('Type upper bound of rectangular domain, e.g.:   pi');  w2 = input('--->  ');
        end       
        nD = 30; u = linspace(u1,u2,nD); w = linspace(w1,w2,nD); [ug,wg] = meshgrid(u,w);
        XX1 = zeros(size(ug)); XX2 = XX1; XX3 = (XX1);
        for j = 1:nD; for k=1:nD; XX1(j,k)=X1(ug(j,k),wg(j,k)); XX2(j,k)=X2(ug(j,k),wg(j,k)); XX3(j,k)=X3(ug(j,k),wg(j,k)); end; end;
        figure(10); PlotHandle = mesh(XX1,XX2,XX3); hold on; % colormap([0 0 0]);  
        set (gca (), 'dataaspectratio', [1 1 1]); % axis equal;    
        np = 100; ttu = linspace(u1,u2,np);  ttw = linspace(w1,w2,np);
        MM = zeros(np, 12);
        for j=1:np; % draw mapped borders of region R in red/blue
            MM(j,1)=X1(ttu(j),w1); MM(j,2)=X2(ttu(j),w1); MM(j,3)=X3(ttu(j),w1);
            MM(j,4)=X1(ttu(j),w2); MM(j,5)=X2(ttu(j),w2); MM(j,6)=X3(ttu(j),w2);
            MM(j,7)=X1(u1,ttw(j)); MM(j,8)=X2(u1,ttw(j)); MM(j,9)=X3(u1,ttw(j));
            MM(j,10)=X1(u2,ttw(j)); MM(j,11)=X2(u2,ttw(j)); MM(j,12)=X3(u2,ttw(j));
        end
        plot3(MM(:,1), MM(:,2), MM(:,3), 'b', 'LineWidth', 3);
        plot3(MM(:,4), MM(:,5), MM(:,6), 'b', 'LineWidth', 3);
        plot3(MM(:,7), MM(:,8), MM(:,9), 'r', 'LineWidth', 3);
        plot3(MM(:,10), MM(:,11), MM(:,12), 'r', 'LineWidth', 3);        
end; %end big switch
disp('Plot done')


%%%%% COMMENTS
%
% In the code there are a lot of "for" loops, which make it slow and
% inefficient, they are necessary to allow non-vectorised function inputs 
% from the user (eg, it takes (1+t)*cos(t) as (1+t).*cos(t)).
%
% Used    set (gca (), 'dataaspectratio', [1 1 1]);  instead of 
% axis equal/square since it behaves nicely in Matlab and doesn't do 
% anything in Octave, the other options might give messy plots in Octave.
%
% Didn't use streamline since there's nothing similar in Octave.


